import asyncio
from contextlib import asynccontextmanager, contextmanager
from dataclasses import asdict, dataclass, field
import json
from typing import AsyncIterator
import uuid
from pydantic import BaseModel, Field
from typing_extensions import Literal

from deno_sandbox.bridge import AsyncBridge
from deno_sandbox.options import Options, get_internal_options
from deno_sandbox.rpc import AsyncRpcClient, RpcClient
from deno_sandbox.sandbox_generated import (
    AsyncSandboxHandle as GeneratedAsyncSandboxHandle,
    SpawnArgs,
    AsyncSandboxProcess as GeneratedAsyncSandboxProcess,
    SandboxHandle,
)
from deno_sandbox.transport import (
    Transport,
    WebSocketTransportFactory,
)


type Mode = Literal["connect", "create"]
type StdIo = Literal["piped", "null"]


@dataclass(frozen=True)
class SandboxCreateOptions:
    env: dict[str, str] | None = None
    """Environment variables to set in the sandbox."""
    memory_mb: int | None = 1024
    """Memory limit for the sandbox in megabytes (default: 1024)."""
    labels: dict[str, str] | None = None
    """
    Labels to set on the sandbox. Up to 5 labels can be specified.
    Each label key must be at most 64 bytes, and each label value must be at most 128 bytes.
    """
    volumes: dict[str, str] | None = None
    """
    Volumes to mount on the sandbox.
    The key is the mount path inside the sandbox, and the value is
    the volume ID or slug
    """


@dataclass(frozen=True)
class SandboxConnectOptions:
    id: uuid.UUID
    """Unique id of the sandbox."""


@dataclass
class SandboxRequestResult:
    revision_id: str


@dataclass
class SandboxDeployOptions:
    path: str | None = None
    entrypoint: str = "main.ts"
    args: list[str] = field(default_factory=list)


@dataclass
class SandboxInfo:
    sandboxId: uuid.UUID
    """The ID of the sandbox."""
    createdAt: str
    """The creation time of the sandbox."""
    region: str
    """The region where the sandbox is running."""


@dataclass
class AppConfig:
    sandbox_id: str
    mode: Mode
    stop_at_ms: int | None
    labels: dict[str, str] | None
    memory_mb: int | None


class Sandbox:
    def __init__(self, options=None):
        self._bridge = AsyncBridge()
        self._async_sandbox = AsyncSandbox(options)

    @contextmanager
    def create(self, options=None):
        async_cm = self._async_sandbox.create(options)
        async_handle = self._bridge.run(async_cm.__aenter__())

        rpc = RpcClient(async_handle._rpc, self._bridge)

        try:
            yield SandboxHandle(rpc, async_handle.id)
        finally:
            self._bridge.run(async_cm.__aexit__(None, None, None))

    def close(self):
        self._bridge.stop()


class AsyncSandboxProcess(GeneratedAsyncSandboxProcess):
    async def spawn(self, args: SpawnArgs) -> RemoteProcess:
        result: SpawnResult = await super().spawn(args)
        return RemoteProcess(result, self._rpc)


class AsyncSandboxHandle(GeneratedAsyncSandboxHandle):
    def __init__(self, rpc: AsyncRpcClient, sandbox_id: str):
        super().__init__(rpc, sandbox_id)
        self.process = AsyncSandboxProcess(rpc)


class AsyncSandbox:
    def __init__(
        self,
        options: Options | None = None,
    ):
        self.__options = get_internal_options(options or Options())
        self._transport_factory = WebSocketTransportFactory()
        self._rpc: AsyncRpcClient | None = None

    async def _init_transport(self, app_config: AppConfig) -> Transport:
        transport = self._transport_factory.create_transport()

        headers = {
            "Authorization": f"Bearer {self.__options.token}",
            "x-denodeploy-config-v2": json.dumps(asdict(app_config)),
        }

        await transport.connect(url=self.__options.sandbox_ws_url, headers=headers)
        return transport

    @asynccontextmanager
    async def create(
        self, options: SandboxCreateOptions | None = None
    ) -> AsyncIterator[AsyncSandboxHandle]:
        """Creates a new sandbox instance."""

        sandbox_id = str(uuid.uuid4())
        app_config = AppConfig(
            sandbox_id=sandbox_id,
            mode="create",
            stop_at_ms=None,
            labels=options.labels if options else None,
            memory_mb=options.memory_mb if options else None,
        )
        transport = await self._init_transport(app_config)

        try:
            self._rpc = AsyncRpcClient(transport)
            yield AsyncSandboxHandle(self._rpc, sandbox_id)
        finally:
            await transport.close()

    @asynccontextmanager
    async def connect(
        self, options: SandboxConnectOptions
    ) -> AsyncIterator[AsyncSandboxHandle]:
        """Connects to an existing sandbox instance."""

        sandbox_id = str(options.id)
        app_config = AppConfig(
            sandbox_id=sandbox_id,
            mode="connect",
            stop_at_ms=None,
            labels=None,
            memory_mb=None,
        )
        transport = await self._init_transport(app_config)

        try:
            rpc = AsyncRpcClient(transport)
            yield AsyncSandboxHandle(rpc, sandbox_id)
        finally:
            await transport.close()


class SpawnResult(BaseModel):
    pid: int
    stdout_stream_id: int | None = Field(default=None, alias="stdoutStreamId")
    stderr_stream_id: int | None = Field(default=None, alias="stderrStreamId")


class ProcessExit(BaseModel):
    code: int


class RemoteProcess:
    def __init__(self, res: SpawnResult, rpc: AsyncRpcClient):
        self.pid = res.pid
        self._stdout_stream_id = res.stdout_stream_id
        self._stderr_stream_id = res.stderr_stream_id
        self.returncode: int | None = None

        self.stdout = asyncio.StreamReader()
        self.stderr = asyncio.StreamReader()
        self._wait_task = asyncio.create_task(
            rpc.call("processWait", {"pid": self.pid})
        )

        rpc._pending_processes[self._stdout_stream_id] = self.stdout
        rpc._pending_processes[self._stderr_stream_id] = self.stderr

    async def wait(self) -> int:
        raw = await self._wait_task
        result = ProcessExit.model_validate(raw)
        self.returncode = result.code
        return self.returncode



from dataclasses import dataclass
import os

from urllib.parse import urlparse

from pydantic_core import Url


@dataclass(frozen=True)
class Options:
    token: str | Url | None = None
    regions: list[str] | None = None


@dataclass
class InternalOptions:
    sandbox_ws_url: Url
    console_url: Url
    token: Url
    regions: list[str]


def get_internal_options(options: Options) -> InternalOptions:
    options = options or Options()

    url = urlparse(
        os.environ.get("DENO_DEPLOY_URL", "https://ams.sandbox-api.deno.net")
    )

    token = options.token or os.environ.get("DENO_DEPLOY_TOKEN", "")

    scheme = url.scheme.replace("http", "ws")
    sandbox_ws_url = Url(f"{scheme}://{url.netloc}/api/v2/sandbox/ws?format=json")

    console_url = os.environ.get("DENO_DEPLOY_CONSOLE_URL", "https://console.deno.com")
    parsed_console_url = urlparse(console_url)

    regions = options.regions or os.environ.get(
        "DENO_AVAILABLE_REGIONS", "ams1,ord"
    ).split(",")

    return InternalOptions(
        console_url=parsed_console_url,
        sandbox_ws_url=sandbox_ws_url,
        token=token,
        regions=regions,
    )



import asyncio
import base64
import json
from typing import Any, Dict, Optional
from pydantic import BaseModel
from websockets import ConnectionClosed

from deno_sandbox.bridge import AsyncBridge
from deno_sandbox.transport import Transport


class RpcRequest(BaseModel):
    method: str
    params: Dict[str, Any]
    id: int
    jsonrpc: str = "2.0"


class RpcResult[T](BaseModel):
    ok: Optional[T] = None
    error: Optional[Any] = None


class RpcResponse[T](BaseModel):
    jsonrpc: str = "2.0"
    result: Optional[RpcResult[T]] = None
    error: Dict[str, Any] | None = None
    id: int


class AsyncRpcClient:
    def __init__(self, transport: Transport):
        self._transport = transport
        self._id = 0
        self._pending_requests: Dict[int, asyncio.Future[Any]] = {}
        self._listen_task = asyncio.create_task(self._listener())
        self._pending_processes: Dict[int, asyncio.StreamReader] = {}

    async def close(self):
        await self._transport.close()

    async def call(self, method: str, params: Dict[str, Any]) -> Any:
        req_id = self._id + 1
        self._id = req_id

        payload = RpcRequest(method=method, params=params, id=req_id)

        loop = asyncio.get_running_loop()
        future = loop.create_future()
        self._pending_requests[req_id] = future

        await self._transport.send(payload.model_dump_json())

        raw_response = await future
        response = RpcResponse[Any](**raw_response)

        if response.error:
            raise Exception(response.error)

        if response.result and response.result.error:
            raise Exception(f"Application Error: {response.result.error}")

        return response.result.ok if response.result else None

    async def _listener(self) -> None:
        try:
            async for raw in self._transport:
                data = json.loads(raw)
                req_id = data.get("id")

                if req_id is not None and req_id in self._pending_requests:
                    future = self._pending_requests.pop(req_id)
                    if not future.done():
                        future.set_result(data)

                elif "method" in data:
                    method = data["method"]
                    params = data.get("params", {})

                    if method == "$sandbox.stream.enqueue":
                        stream_id = params.get("streamId")
                        chunk = base64.b64decode(params.get("data", ""))
                        stream = self._pending_processes.get(stream_id)
                        if stream:
                            stream.feed_data(chunk)
                    elif method == "$sandbox.stream.end":
                        stream_id = params.get("streamId")
                        stream = self._pending_processes.get(stream_id)
                        if stream:
                            stream.feed_eof()
                            del self._pending_processes[stream_id]

        except ConnectionClosed:
            pass
        except Exception as e:
            for future in self._pending_requests.values():
                if not future.done():
                    future.set_exception(e)


class RpcClient:
    def __init__(self, async_client: AsyncRpcClient, bridge: AsyncBridge):
        self._async_client = async_client
        self._bridge = bridge

    def call(self, method: str, params: Dict[str, Any]) -> Any:
        return self._bridge.run(self._async_client.call(method, params))

    def close(self):
        self._bridge.run(self._async_client.close())



import httpx
from pydantic import HttpUrl

from deno_sandbox.options import Options, get_internal_options


class AsyncConsoleClient:
    def __init__(self, options: Options):
        self.__options = get_internal_options(options or Options())

    async def post(self, path: str, data: any) -> dict:
        async with httpx.AsyncClient() as client:
            req_url = HttpUrl(self.__options.console_url, path=path)
            headers = {
                "Content-Type": "application/json",
                "Authorization": f"Bearer {self.__options.token}",
            }
            response = await client.post(req_url, headers=headers, json=data)
            response.raise_for_status()
            return await response.json()

    async def get(self, path: str, search: dict | None = None) -> dict:
        async with httpx.AsyncClient() as client:
            req_url = HttpUrl(self.__options.console_url, path=path, search=search)
            headers = {
                "Content-Type": "application/json",
                "Authorization": f"Bearer {self.__options.token}",
            }
            response = await client.get(req_url, headers=headers)
            response.raise_for_status()
            return await response.json()

    async def delete(self, path: str) -> None:
        async with httpx.AsyncClient() as client:
            req_url = HttpUrl(self.__options.console_url, path=path)
            headers = {
                "Content-Type": "application/json",
                "Authorization": f"Bearer {self.__options.token}",
            }
            response = await client.delete(req_url, headers=headers)
            response.raise_for_status()



# ATTENTION: This file is auto-generated. Do not edit it manually.

from pydantic import BaseModel, ConfigDict, validate_call
from pydantic.alias_generators import to_camel
from typing import Any
from typing_extensions import Literal

from deno_sandbox.rpc import AsyncRpcClient, RpcClient


class AbortArgs(BaseModel):
    model_config = ConfigDict(alias_generator=to_camel)
    abort_id: int


class ReadFileArgs(BaseModel):
    model_config = ConfigDict(alias_generator=to_camel)
    path: str
    abort_id: int | None = None


class ReadTextFileArgs(BaseModel):
    model_config = ConfigDict(alias_generator=to_camel)
    path: str
    abort_id: int | None = None


class ReadDirArgs(BaseModel):
    model_config = ConfigDict(alias_generator=to_camel)
    path: str


class RemoveOptions(BaseModel):
    model_config = ConfigDict(alias_generator=to_camel)
    recursive: bool | None = None


class RemoveArgs(BaseModel):
    model_config = ConfigDict(alias_generator=to_camel)
    path: str
    options: RemoveOptions | None = None


class MkdirOptions(BaseModel):
    model_config = ConfigDict(alias_generator=to_camel)
    recursive: bool | None = None
    mode: int | None = None


class MkdirArgs(BaseModel):
    model_config = ConfigDict(alias_generator=to_camel)
    path: str
    options: MkdirOptions | None = None


class RenameArgs(BaseModel):
    model_config = ConfigDict(alias_generator=to_camel)
    old_path: str
    new_path: str


class CopyFileArgs(BaseModel):
    model_config = ConfigDict(alias_generator=to_camel)
    from_path: str
    to_path: str


class LinkArgs(BaseModel):
    model_config = ConfigDict(alias_generator=to_camel)
    target: str
    path: str


class LstatArgs(BaseModel):
    model_config = ConfigDict(alias_generator=to_camel)
    path: str


class LstatResult(BaseModel):
    model_config = ConfigDict(alias_generator=to_camel)
    is_file: bool
    is_directory: bool
    is_symlink: bool
    size: int
    mtime: str
    atime: str
    birthtime: str
    ctime: str
    mode: int
    dev: int
    ino: int
    nlink: int
    uid: int
    gid: int
    rdev: int
    blksize: int
    blocks: int
    is_block_device: bool
    is_char_device: bool
    is_fifo: bool
    is_socket: bool


class MakeTempDirOptions(BaseModel):
    model_config = ConfigDict(alias_generator=to_camel)
    dir: str | None = None
    prefix: str | None = None
    suffix: str | None = None


class MakeTempDirArgs(BaseModel):
    model_config = ConfigDict(alias_generator=to_camel)
    options: MakeTempDirOptions | None = None


class MakeTempFileOptions(BaseModel):
    model_config = ConfigDict(alias_generator=to_camel)
    dir: str | None = None
    prefix: str | None = None
    suffix: str | None = None


class MakeTempFileArgs(BaseModel):
    model_config = ConfigDict(alias_generator=to_camel)
    options: MakeTempFileOptions | None = None


class ReadLinkArgs(BaseModel):
    model_config = ConfigDict(alias_generator=to_camel)
    path: str


class RealPathArgs(BaseModel):
    model_config = ConfigDict(alias_generator=to_camel)
    path: str


class SymlinkOptions(BaseModel):
    model_config = ConfigDict(alias_generator=to_camel)
    type: Literal["file", "dir", "junction"] | None = None


class SymlinkArgs(BaseModel):
    model_config = ConfigDict(alias_generator=to_camel)
    target: str
    path: str
    options: SymlinkOptions | None = None


class TruncateArgs(BaseModel):
    model_config = ConfigDict(alias_generator=to_camel)
    name: str
    len: int | None = None


class UmaskArgs(BaseModel):
    model_config = ConfigDict(alias_generator=to_camel)
    mask: int | None = None


class UtimeArgs(BaseModel):
    model_config = ConfigDict(alias_generator=to_camel)
    path: str
    atime: str
    mtime: str


class FileOpenOptions(BaseModel):
    model_config = ConfigDict(alias_generator=to_camel)
    read: bool | None = None
    write: bool | None = None
    append: bool | None = None
    truncate: bool | None = None
    create: bool | None = None
    create_new: bool | None = None
    mode: int | None = None


class OpenArgs(BaseModel):
    model_config = ConfigDict(alias_generator=to_camel)
    path: str
    options: FileOpenOptions | None = None


class OpenResult(BaseModel):
    model_config = ConfigDict(alias_generator=to_camel)
    file_handle_id: int


class CreateArgs(BaseModel):
    model_config = ConfigDict(alias_generator=to_camel)
    path: str


class CreateResult(BaseModel):
    model_config = ConfigDict(alias_generator=to_camel)
    file_handle_id: int


class FileCloseArgs(BaseModel):
    model_config = ConfigDict(alias_generator=to_camel)
    file_handle_id: int


class FileLockArgs(BaseModel):
    model_config = ConfigDict(alias_generator=to_camel)
    file_handle_id: int
    exclusive: bool | None = None


class FileReadArgs(BaseModel):
    model_config = ConfigDict(alias_generator=to_camel)
    file_handle_id: int
    length: int


class FileReadResult(BaseModel):
    model_config = ConfigDict(alias_generator=to_camel)
    data: str | None = None


class FileSeekArgs(BaseModel):
    model_config = ConfigDict(alias_generator=to_camel)
    file_handle_id: int
    offset: int | str
    whence: int


class FileSeekResult(BaseModel):
    model_config = ConfigDict(alias_generator=to_camel)
    position: int


class FileStatArgs(BaseModel):
    model_config = ConfigDict(alias_generator=to_camel)
    file_handle_id: int


class FileStatResult(BaseModel):
    model_config = ConfigDict(alias_generator=to_camel)
    is_file: bool
    is_directory: bool
    is_symlink: bool
    size: int
    mtime: str
    atime: str
    birthtime: str
    ctime: str
    mode: int
    dev: int
    ino: int
    nlink: int
    uid: int
    gid: int
    rdev: int
    blksize: int
    blocks: int
    is_block_device: bool
    is_char_device: bool
    is_fifo: bool
    is_socket: bool


class FileSyncArgs(BaseModel):
    model_config = ConfigDict(alias_generator=to_camel)
    file_handle_id: int


class FileSyncDataArgs(BaseModel):
    model_config = ConfigDict(alias_generator=to_camel)
    file_handle_id: int


class FileTruncateArgs(BaseModel):
    model_config = ConfigDict(alias_generator=to_camel)
    file_handle_id: int
    len: int | None = None


class FileUnlockArgs(BaseModel):
    model_config = ConfigDict(alias_generator=to_camel)
    file_handle_id: int


class FileUtimeArgs(BaseModel):
    model_config = ConfigDict(alias_generator=to_camel)
    file_handle_id: int
    atime: str
    mtime: str


class FileWriteArgs(BaseModel):
    model_config = ConfigDict(alias_generator=to_camel)
    file_handle_id: int
    data: str


class FileWriteResult(BaseModel):
    model_config = ConfigDict(alias_generator=to_camel)
    bytes_written: int


class WalkArgs(BaseModel):
    model_config = ConfigDict(alias_generator=to_camel)
    path: str
    max_depth: int | None = None
    include_files: bool | None = None
    include_dirs: bool | None = None
    include_symlinks: bool | None = None
    follow_symlinks: bool | None = None
    canonicalize: bool | None = None
    exts: list[str] | None = None
    match: list[str] | None = None
    skip: list[str] | None = None


class ExpandGlobArgs(BaseModel):
    model_config = ConfigDict(alias_generator=to_camel)
    glob: str
    extended: bool | None = None
    globstar: bool | None = None
    case_insensitive: bool | None = None
    root: str | None = None
    exclude: list[str] | None = None
    include_dirs: bool | None = None
    follow_symlinks: bool | None = None
    canonicalize: bool | None = None


class SpawnArgs(BaseModel):
    model_config = ConfigDict(alias_generator=to_camel)
    command: str
    args: list[str] | None = None
    env: dict[str, str] | None = None
    clear_env: bool | None = None
    cwd: str | None = None
    stdin_stream_id: int | None = None
    stdout: Literal["piped", "null"] = "piped"
    stderr: Literal["piped", "null"] = "piped"


class SpawnResult(BaseModel):
    model_config = ConfigDict(alias_generator=to_camel)
    pid: int
    stdout_stream_id: int | None = None
    stderr_stream_id: int | None = None


class ProcessWaitArgs(BaseModel):
    model_config = ConfigDict(alias_generator=to_camel)
    pid: int
    vscode: bool | None = None


class ProcessWaitResult(BaseModel):
    model_config = ConfigDict(alias_generator=to_camel)
    success: bool
    code: int
    signal: str | None = None


class ProcessKillArgs(BaseModel):
    model_config = ConfigDict(alias_generator=to_camel)
    pid: int
    signal: str | None = None
    vscode: bool | None = None


class RunResult(BaseModel):
    model_config = ConfigDict(alias_generator=to_camel)
    pid: int
    stdout_stream_id: int | None = None
    stderr_stream_id: int | None = None


class DenoHttpWaitArgs(BaseModel):
    model_config = ConfigDict(alias_generator=to_camel)
    pid: int


class SpawnDenoReplArgs(BaseModel):
    model_config = ConfigDict(alias_generator=to_camel)
    env: dict[str, str] | None = None
    clear_env: bool | None = None
    cwd: str | None = None
    stdin_stream_id: int | None = None
    stdout: Literal["piped", "null"] = "piped"
    stderr: Literal["piped", "null"] = "piped"
    script_args: list[str] | None = None


class SpawnDenoReplResult(BaseModel):
    model_config = ConfigDict(alias_generator=to_camel)
    pid: int
    stdout_stream_id: int | None = None
    stderr_stream_id: int | None = None


class DenoReplCloseArgs(BaseModel):
    model_config = ConfigDict(alias_generator=to_camel)
    pid: int


class DenoReplEvalArgs(BaseModel):
    model_config = ConfigDict(alias_generator=to_camel)
    pid: int
    code: str


class DenoReplCallArgs(BaseModel):
    model_config = ConfigDict(alias_generator=to_camel)
    pid: int
    fn: str
    args: list[Any]


class FetchArgs(BaseModel):
    model_config = ConfigDict(alias_generator=to_camel)
    url: str
    method: str
    headers: dict[str, str]
    redirect: Literal["follow", "manual"]
    abort_id: int | None = None
    body_stream_id: int | None = None
    pid: int | None = None


class FetchResult(BaseModel):
    model_config = ConfigDict(alias_generator=to_camel)
    status: int
    status_text: str
    headers: dict[str, str]
    body_stream_id: int | None = None


class VSCodeConnectArgs(BaseModel):
    model_config = ConfigDict(alias_generator=to_camel)
    path: str
    extensions: list[str]
    env: dict[str, str] | None = None
    preview: str | None = None
    disable_stop_button: bool | None = None
    editor_settings: dict[str, Any] | None = None


class VSCodeConnectResult(BaseModel):
    model_config = ConfigDict(alias_generator=to_camel)
    pid: int
    port: int
    stdout_stream_id: int
    stderr_stream_id: int


class EnvGetArgs(BaseModel):
    model_config = ConfigDict(alias_generator=to_camel)
    key: str


class EnvSetArgs(BaseModel):
    model_config = ConfigDict(alias_generator=to_camel)
    key: str
    value: str


class EnvDeleteArgs(BaseModel):
    model_config = ConfigDict(alias_generator=to_camel)
    key: str


class WriteFileOptions(BaseModel):
    model_config = ConfigDict(alias_generator=to_camel)
    create: bool | None = None
    append: bool | None = None
    create_new: bool | None = None
    mode: int | None = None


class WriteFileWithContent(BaseModel):
    model_config = ConfigDict(alias_generator=to_camel)
    path: str
    abort_id: int | None = None
    options: WriteFileOptions | None = None
    content: str


class WriteFileWithStream(BaseModel):
    model_config = ConfigDict(alias_generator=to_camel)
    path: str
    abort_id: int | None = None
    options: WriteFileOptions | None = None
    content_stream_id: int


class WriteTextFileWithContent(BaseModel):
    model_config = ConfigDict(alias_generator=to_camel)
    path: str
    abort_id: int | None = None
    options: WriteFileOptions | None = None
    content: str


class WriteTextFileWithStream(BaseModel):
    model_config = ConfigDict(alias_generator=to_camel)
    path: str
    abort_id: int | None = None
    options: WriteFileOptions | None = None
    content_stream_id: int


class ReadDirEntry(BaseModel):
    model_config = ConfigDict(alias_generator=to_camel)
    name: str
    is_file: bool
    is_directory: bool
    is_symlink: bool


class SpawnDenoByEntrypoint(BaseModel):
    model_config = ConfigDict(alias_generator=to_camel)
    env: dict[str, str] | None = None
    clear_env: bool | None = None
    cwd: str | None = None
    stdin_stream_id: int | None = None
    stdout: Literal["piped", "null"] = "piped"
    stderr: Literal["piped", "null"] = "piped"
    script_args: list[str] | None = None
    entrypoint: str


class SpawnDenoByCode(BaseModel):
    model_config = ConfigDict(alias_generator=to_camel)
    env: dict[str, str] | None = None
    clear_env: bool | None = None
    cwd: str | None = None
    stdin_stream_id: int | None = None
    stdout: Literal["piped", "null"] = "piped"
    stderr: Literal["piped", "null"] = "piped"
    script_args: list[str] | None = None
    code: str
    extension: Literal["js", "cjs", "mjs", "ts", "cts", "mts", "jsx", "tsx"]


class ExposeHttpByPort(BaseModel):
    model_config = ConfigDict(alias_generator=to_camel)
    domain: str
    port: int


class ExposeHttpByPid(BaseModel):
    model_config = ConfigDict(alias_generator=to_camel)
    domain: str
    pid: int


class SandboxHandle:
    def __init__(self, rpc: RpcClient, id: str):
        self.id = id
        self._rpc = rpc
        self.fs = SandboxFs(rpc)
        self.process = SandboxProcess(rpc)
        self.deno = SandboxDeno(rpc)
        self.net = SandboxNet(rpc)
        self.vscode = SandboxVscode(rpc)
        self.env = SandboxEnv(rpc)

    @validate_call(validate_return=True)
    def abort(self, args: AbortArgs) -> None:
        value = args.model_dump()
        self._rpc.call("abort", value)


class SandboxFs:
    def __init__(self, rpc: RpcClient):
        self._rpc = rpc

    @validate_call(validate_return=True)
    def read_file(self, args: ReadFileArgs) -> None:
        """Reads the entire contents of a file as bytes."""

        value = args.model_dump()
        self._rpc.call("readFile", value)

    @validate_call(validate_return=True)
    def read_text_file(self, args: ReadTextFileArgs) -> str:
        """Reads the entire contents of a file as an UTF-8 decoded string."""

        value = args.model_dump()
        return self._rpc.call("readTextFile", value)

    @validate_call(validate_return=True)
    def write_file(self, args: WriteFileWithContent | WriteFileWithStream) -> None:
        """Write bytes to file. Creates a new file if needed. Existing files will be overwritten."""

        value = args.model_dump()
        self._rpc.call("writeFile", value)

    @validate_call(validate_return=True)
    def write_text_file(
        self, args: WriteTextFileWithContent | WriteTextFileWithStream
    ) -> None:
        """Write text to file. Creates a new file if needed. Existing files will be overwritten."""

        value = args.model_dump()
        self._rpc.call("writeTextFile", value)

    @validate_call(validate_return=True)
    def read_dir(self, args: ReadDirArgs) -> list[ReadDirEntry]:
        """Read the directory entries at the given path."""

        value = args.model_dump()
        return self._rpc.call("readDir", value)

    @validate_call(validate_return=True)
    def remove(self, args: RemoveArgs) -> None:
        """Remove the file or directory at the given path."""

        value = args.model_dump()
        self._rpc.call("remove", value)

    @validate_call(validate_return=True)
    def mkdir(self, args: MkdirArgs) -> None:
        """Create a new directory at the specified path."""

        value = args.model_dump()
        self._rpc.call("mkdir", value)

    @validate_call(validate_return=True)
    def rename(self, args: RenameArgs) -> None:
        """Rename (move) a file or directory."""

        value = args.model_dump()
        self._rpc.call("rename", value)

    @validate_call(validate_return=True)
    def copy_file(self, args: CopyFileArgs) -> None:
        """Copy a file from one location to another."""

        value = args.model_dump()
        self._rpc.call("copyFile", value)

    @validate_call(validate_return=True)
    def link(self, args: LinkArgs) -> None:
        """Create a hard link pointing to an existing file."""

        value = args.model_dump()
        self._rpc.call("link", value)

    @validate_call(validate_return=True)
    def lstat(self, args: LstatArgs) -> LstatResult:
        """Return file information about a file or directory symlink."""

        value = args.model_dump()
        return self._rpc.call("lstat", value)

    @validate_call(validate_return=True)
    def make_temp_dir(self, args: MakeTempDirArgs) -> str:
        """Create a new temporary directory."""

        value = args.model_dump()
        return self._rpc.call("makeTempDir", value)

    @validate_call(validate_return=True)
    def make_temp_file(self, args: MakeTempFileArgs) -> str:
        """Create a new temporary file."""

        value = args.model_dump()
        return self._rpc.call("makeTempFile", value)

    @validate_call(validate_return=True)
    def read_link(self, args: ReadLinkArgs) -> str:
        """Read the target of a symbolic link."""

        value = args.model_dump()
        return self._rpc.call("readLink", value)

    @validate_call(validate_return=True)
    def real_path(self, args: RealPathArgs) -> str:
        """Return the canonicalized absolute pathname."""

        value = args.model_dump()
        return self._rpc.call("realPath", value)

    @validate_call(validate_return=True)
    def symlink(self, args: SymlinkArgs) -> None:
        """Create a symbolic link."""

        value = args.model_dump()
        self._rpc.call("symlink", value)

    @validate_call(validate_return=True)
    def truncate(self, args: TruncateArgs) -> None:
        """Truncate or extend the specified file to reach a given size."""

        value = args.model_dump()
        self._rpc.call("truncate", value)

    @validate_call(validate_return=True)
    def umask(self, args: UmaskArgs) -> int:
        """Sets the process's file mode creation mask."""

        value = args.model_dump()
        return self._rpc.call("umask", value)

    @validate_call(validate_return=True)
    def utime(self, args: UtimeArgs) -> None:
        """Change the access and modification times of a file."""

        value = args.model_dump()
        self._rpc.call("utime", value)

    @validate_call(validate_return=True)
    def open(self, args: OpenArgs) -> OpenResult:
        """Open a file and return a file handle id."""

        value = args.model_dump()
        return self._rpc.call("open", value)

    @validate_call(validate_return=True)
    def create(self, args: CreateArgs) -> CreateResult:
        """Create a new file and return a file handle id."""

        value = args.model_dump()
        return self._rpc.call("create", value)

    @validate_call(validate_return=True)
    def file_close(self, args: FileCloseArgs) -> None:
        """Close the specified file handle."""

        value = args.model_dump()
        self._rpc.call("fileClose", value)

    @validate_call(validate_return=True)
    def file_lock(self, args: FileLockArgs) -> None:
        value = args.model_dump()
        self._rpc.call("fileLock", value)

    @validate_call(validate_return=True)
    def file_read(self, args: FileReadArgs) -> FileReadResult:
        value = args.model_dump()
        return self._rpc.call("fileRead", value)

    @validate_call(validate_return=True)
    def file_seek(self, args: FileSeekArgs) -> FileSeekResult:
        value = args.model_dump()
        return self._rpc.call("fileSeek", value)

    @validate_call(validate_return=True)
    def file_stat(self, args: FileStatArgs) -> FileStatResult:
        value = args.model_dump()
        return self._rpc.call("fileStat", value)

    @validate_call(validate_return=True)
    def file_sync(self, args: FileSyncArgs) -> None:
        value = args.model_dump()
        self._rpc.call("fileSync", value)

    @validate_call(validate_return=True)
    def file_sync_data(self, args: FileSyncDataArgs) -> None:
        value = args.model_dump()
        self._rpc.call("fileSyncData", value)

    @validate_call(validate_return=True)
    def file_truncate(self, args: FileTruncateArgs) -> None:
        value = args.model_dump()
        self._rpc.call("fileTruncate", value)

    @validate_call(validate_return=True)
    def file_unlock(self, args: FileUnlockArgs) -> None:
        value = args.model_dump()
        self._rpc.call("fileUnlock", value)

    @validate_call(validate_return=True)
    def file_utime(self, args: FileUtimeArgs) -> None:
        value = args.model_dump()
        self._rpc.call("fileUtime", value)

    @validate_call(validate_return=True)
    def file_write(self, args: FileWriteArgs) -> FileWriteResult:
        value = args.model_dump()
        return self._rpc.call("fileWrite", value)

    @validate_call(validate_return=True)
    def walk(self, args: WalkArgs) -> int:
        value = args.model_dump()
        return self._rpc.call("walk", value)

    @validate_call(validate_return=True)
    def expand_glob(self, args: ExpandGlobArgs) -> int:
        value = args.model_dump()
        return self._rpc.call("expandGlob", value)


class SandboxProcess:
    def __init__(self, rpc: RpcClient):
        self._rpc = rpc

    @validate_call(validate_return=True)
    def spawn(self, args: SpawnArgs) -> SpawnResult:
        value = args.model_dump()
        return self._rpc.call("spawn", value)

    @validate_call(validate_return=True)
    def wait(self, args: ProcessWaitArgs) -> ProcessWaitResult:
        value = args.model_dump()
        return self._rpc.call("processWait", value)

    @validate_call(validate_return=True)
    def kill(self, args: ProcessKillArgs) -> None:
        value = args.model_dump()
        self._rpc.call("processKill", value)


class SandboxDeno:
    def __init__(self, rpc: RpcClient):
        self._rpc = rpc

    @validate_call(validate_return=True)
    def run(self, args: SpawnDenoByEntrypoint | SpawnDenoByCode) -> RunResult:
        value = args.model_dump()
        return self._rpc.call("spawnDeno", value)

    @validate_call(validate_return=True)
    def deno_http_wait(self, args: DenoHttpWaitArgs) -> bool:
        value = args.model_dump()
        return self._rpc.call("denoHttpWait", value)

    @validate_call(validate_return=True)
    def spawn_deno_repl(self, args: SpawnDenoReplArgs) -> SpawnDenoReplResult:
        value = args.model_dump()
        return self._rpc.call("spawnDenoRepl", value)

    @validate_call(validate_return=True)
    def deno_repl_close(self, args: DenoReplCloseArgs) -> None:
        value = args.model_dump()
        self._rpc.call("denoReplClose", value)

    @validate_call(validate_return=True)
    def deno_repl_eval(self, args: DenoReplEvalArgs) -> Any:
        value = args.model_dump()
        return self._rpc.call("denoReplEval", value)

    @validate_call(validate_return=True)
    def deno_repl_call(self, args: DenoReplCallArgs) -> Any:
        value = args.model_dump()
        return self._rpc.call("denoReplCall", value)


class SandboxNet:
    def __init__(self, rpc: RpcClient):
        self._rpc = rpc

    @validate_call(validate_return=True)
    def fetch(self, args: FetchArgs) -> FetchResult:
        value = args.model_dump()
        return self._rpc.call("fetch", value)

    @validate_call(validate_return=True)
    def expose_http(self, args: ExposeHttpByPort | ExposeHttpByPid) -> None:
        value = args.model_dump()
        self._rpc.call("exposeHttp", value)


class SandboxVscode:
    def __init__(self, rpc: RpcClient):
        self._rpc = rpc

    @validate_call(validate_return=True)
    def connect(self, args: VSCodeConnectArgs) -> VSCodeConnectResult:
        value = args.model_dump()
        return self._rpc.call("connect", value)


class SandboxEnv:
    def __init__(self, rpc: RpcClient):
        self._rpc = rpc

    @validate_call(validate_return=True)
    def get(self, args: EnvGetArgs) -> str:
        value = args.model_dump()
        return self._rpc.call("envGet", value)

    @validate_call(validate_return=True)
    def set(self, args: EnvSetArgs) -> None:
        value = args.model_dump()
        self._rpc.call("envSet", value)

    @validate_call(validate_return=True)
    def to_object(self, args: None) -> dict[str, str]:
        value = args.model_dump()
        return self._rpc.call("envToObject", value)

    @validate_call(validate_return=True)
    def delete(self, args: EnvDeleteArgs) -> None:
        value = args.model_dump()
        self._rpc.call("envDelete", value)


class AsyncSandboxHandle:
    def __init__(self, rpc: AsyncRpcClient, id: str):
        self.id = id
        self._rpc = rpc
        self.fs = AsyncSandboxFs(rpc)
        self.process = AsyncSandboxProcess(rpc)
        self.deno = AsyncSandboxDeno(rpc)
        self.net = AsyncSandboxNet(rpc)
        self.vscode = AsyncSandboxVscode(rpc)
        self.env = AsyncSandboxEnv(rpc)

    @validate_call(validate_return=True)
    async def abort(self, args: AbortArgs) -> None:
        value = args.model_dump()
        await self._rpc.call("abort", value)


class AsyncSandboxFs:
    def __init__(self, rpc: AsyncRpcClient):
        self._rpc = rpc

    @validate_call(validate_return=True)
    async def read_file(self, args: ReadFileArgs) -> None:
        """Reads the entire contents of a file as bytes."""

        value = args.model_dump()
        await self._rpc.call("readFile", value)

    @validate_call(validate_return=True)
    async def read_text_file(self, args: ReadTextFileArgs) -> str:
        """Reads the entire contents of a file as an UTF-8 decoded string."""

        value = args.model_dump()
        return await self._rpc.call("readTextFile", value)

    @validate_call(validate_return=True)
    async def write_file(
        self, args: WriteFileWithContent | WriteFileWithStream
    ) -> None:
        """Write bytes to file. Creates a new file if needed. Existing files will be overwritten."""

        value = args.model_dump()
        await self._rpc.call("writeFile", value)

    @validate_call(validate_return=True)
    async def write_text_file(
        self, args: WriteTextFileWithContent | WriteTextFileWithStream
    ) -> None:
        """Write text to file. Creates a new file if needed. Existing files will be overwritten."""

        value = args.model_dump()
        await self._rpc.call("writeTextFile", value)

    @validate_call(validate_return=True)
    async def read_dir(self, args: ReadDirArgs) -> list[ReadDirEntry]:
        """Read the directory entries at the given path."""

        value = args.model_dump()
        return await self._rpc.call("readDir", value)

    @validate_call(validate_return=True)
    async def remove(self, args: RemoveArgs) -> None:
        """Remove the file or directory at the given path."""

        value = args.model_dump()
        await self._rpc.call("remove", value)

    @validate_call(validate_return=True)
    async def mkdir(self, args: MkdirArgs) -> None:
        """Create a new directory at the specified path."""

        value = args.model_dump()
        await self._rpc.call("mkdir", value)

    @validate_call(validate_return=True)
    async def rename(self, args: RenameArgs) -> None:
        """Rename (move) a file or directory."""

        value = args.model_dump()
        await self._rpc.call("rename", value)

    @validate_call(validate_return=True)
    async def copy_file(self, args: CopyFileArgs) -> None:
        """Copy a file from one location to another."""

        value = args.model_dump()
        await self._rpc.call("copyFile", value)

    @validate_call(validate_return=True)
    async def link(self, args: LinkArgs) -> None:
        """Create a hard link pointing to an existing file."""

        value = args.model_dump()
        await self._rpc.call("link", value)

    @validate_call(validate_return=True)
    async def lstat(self, args: LstatArgs) -> LstatResult:
        """Return file information about a file or directory symlink."""

        value = args.model_dump()
        return await self._rpc.call("lstat", value)

    @validate_call(validate_return=True)
    async def make_temp_dir(self, args: MakeTempDirArgs) -> str:
        """Create a new temporary directory."""

        value = args.model_dump()
        return await self._rpc.call("makeTempDir", value)

    @validate_call(validate_return=True)
    async def make_temp_file(self, args: MakeTempFileArgs) -> str:
        """Create a new temporary file."""

        value = args.model_dump()
        return await self._rpc.call("makeTempFile", value)

    @validate_call(validate_return=True)
    async def read_link(self, args: ReadLinkArgs) -> str:
        """Read the target of a symbolic link."""

        value = args.model_dump()
        return await self._rpc.call("readLink", value)

    @validate_call(validate_return=True)
    async def real_path(self, args: RealPathArgs) -> str:
        """Return the canonicalized absolute pathname."""

        value = args.model_dump()
        return await self._rpc.call("realPath", value)

    @validate_call(validate_return=True)
    async def symlink(self, args: SymlinkArgs) -> None:
        """Create a symbolic link."""

        value = args.model_dump()
        await self._rpc.call("symlink", value)

    @validate_call(validate_return=True)
    async def truncate(self, args: TruncateArgs) -> None:
        """Truncate or extend the specified file to reach a given size."""

        value = args.model_dump()
        await self._rpc.call("truncate", value)

    @validate_call(validate_return=True)
    async def umask(self, args: UmaskArgs) -> int:
        """Sets the process's file mode creation mask."""

        value = args.model_dump()
        return await self._rpc.call("umask", value)

    @validate_call(validate_return=True)
    async def utime(self, args: UtimeArgs) -> None:
        """Change the access and modification times of a file."""

        value = args.model_dump()
        await self._rpc.call("utime", value)

    @validate_call(validate_return=True)
    async def open(self, args: OpenArgs) -> OpenResult:
        """Open a file and return a file handle id."""

        value = args.model_dump()
        return await self._rpc.call("open", value)

    @validate_call(validate_return=True)
    async def create(self, args: CreateArgs) -> CreateResult:
        """Create a new file and return a file handle id."""

        value = args.model_dump()
        return await self._rpc.call("create", value)

    @validate_call(validate_return=True)
    async def file_close(self, args: FileCloseArgs) -> None:
        """Close the specified file handle."""

        value = args.model_dump()
        await self._rpc.call("fileClose", value)

    @validate_call(validate_return=True)
    async def file_lock(self, args: FileLockArgs) -> None:
        value = args.model_dump()
        await self._rpc.call("fileLock", value)

    @validate_call(validate_return=True)
    async def file_read(self, args: FileReadArgs) -> FileReadResult:
        value = args.model_dump()
        return await self._rpc.call("fileRead", value)

    @validate_call(validate_return=True)
    async def file_seek(self, args: FileSeekArgs) -> FileSeekResult:
        value = args.model_dump()
        return await self._rpc.call("fileSeek", value)

    @validate_call(validate_return=True)
    async def file_stat(self, args: FileStatArgs) -> FileStatResult:
        value = args.model_dump()
        return await self._rpc.call("fileStat", value)

    @validate_call(validate_return=True)
    async def file_sync(self, args: FileSyncArgs) -> None:
        value = args.model_dump()
        await self._rpc.call("fileSync", value)

    @validate_call(validate_return=True)
    async def file_sync_data(self, args: FileSyncDataArgs) -> None:
        value = args.model_dump()
        await self._rpc.call("fileSyncData", value)

    @validate_call(validate_return=True)
    async def file_truncate(self, args: FileTruncateArgs) -> None:
        value = args.model_dump()
        await self._rpc.call("fileTruncate", value)

    @validate_call(validate_return=True)
    async def file_unlock(self, args: FileUnlockArgs) -> None:
        value = args.model_dump()
        await self._rpc.call("fileUnlock", value)

    @validate_call(validate_return=True)
    async def file_utime(self, args: FileUtimeArgs) -> None:
        value = args.model_dump()
        await self._rpc.call("fileUtime", value)

    @validate_call(validate_return=True)
    async def file_write(self, args: FileWriteArgs) -> FileWriteResult:
        value = args.model_dump()
        return await self._rpc.call("fileWrite", value)

    @validate_call(validate_return=True)
    async def walk(self, args: WalkArgs) -> int:
        value = args.model_dump()
        return await self._rpc.call("walk", value)

    @validate_call(validate_return=True)
    async def expand_glob(self, args: ExpandGlobArgs) -> int:
        value = args.model_dump()
        return await self._rpc.call("expandGlob", value)


class AsyncSandboxProcess:
    def __init__(self, rpc: AsyncRpcClient):
        self._rpc = rpc

    @validate_call(validate_return=True)
    async def spawn(self, args: SpawnArgs) -> SpawnResult:
        value = args.model_dump()
        return await self._rpc.call("spawn", value)

    @validate_call(validate_return=True)
    async def wait(self, args: ProcessWaitArgs) -> ProcessWaitResult:
        value = args.model_dump()
        return await self._rpc.call("processWait", value)

    @validate_call(validate_return=True)
    async def kill(self, args: ProcessKillArgs) -> None:
        value = args.model_dump()
        await self._rpc.call("processKill", value)


class AsyncSandboxDeno:
    def __init__(self, rpc: AsyncRpcClient):
        self._rpc = rpc

    @validate_call(validate_return=True)
    async def run(self, args: SpawnDenoByEntrypoint | SpawnDenoByCode) -> RunResult:
        value = args.model_dump()
        return await self._rpc.call("spawnDeno", value)

    @validate_call(validate_return=True)
    async def deno_http_wait(self, args: DenoHttpWaitArgs) -> bool:
        value = args.model_dump()
        return await self._rpc.call("denoHttpWait", value)

    @validate_call(validate_return=True)
    async def spawn_deno_repl(self, args: SpawnDenoReplArgs) -> SpawnDenoReplResult:
        value = args.model_dump()
        return await self._rpc.call("spawnDenoRepl", value)

    @validate_call(validate_return=True)
    async def deno_repl_close(self, args: DenoReplCloseArgs) -> None:
        value = args.model_dump()
        await self._rpc.call("denoReplClose", value)

    @validate_call(validate_return=True)
    async def deno_repl_eval(self, args: DenoReplEvalArgs) -> Any:
        value = args.model_dump()
        return await self._rpc.call("denoReplEval", value)

    @validate_call(validate_return=True)
    async def deno_repl_call(self, args: DenoReplCallArgs) -> Any:
        value = args.model_dump()
        return await self._rpc.call("denoReplCall", value)


class AsyncSandboxNet:
    def __init__(self, rpc: AsyncRpcClient):
        self._rpc = rpc

    @validate_call(validate_return=True)
    async def fetch(self, args: FetchArgs) -> FetchResult:
        value = args.model_dump()
        return await self._rpc.call("fetch", value)

    @validate_call(validate_return=True)
    async def expose_http(self, args: ExposeHttpByPort | ExposeHttpByPid) -> None:
        value = args.model_dump()
        await self._rpc.call("exposeHttp", value)


class AsyncSandboxVscode:
    def __init__(self, rpc: AsyncRpcClient):
        self._rpc = rpc

    @validate_call(validate_return=True)
    async def connect(self, args: VSCodeConnectArgs) -> VSCodeConnectResult:
        value = args.model_dump()
        return await self._rpc.call("connect", value)


class AsyncSandboxEnv:
    def __init__(self, rpc: AsyncRpcClient):
        self._rpc = rpc

    @validate_call(validate_return=True)
    async def get(self, args: EnvGetArgs) -> str:
        value = args.model_dump()
        return await self._rpc.call("envGet", value)

    @validate_call(validate_return=True)
    async def set(self, args: EnvSetArgs) -> None:
        value = args.model_dump()
        await self._rpc.call("envSet", value)

    @validate_call(validate_return=True)
    async def to_object(self, args: None) -> dict[str, str]:
        value = args.model_dump()
        return await self._rpc.call("envToObject", value)

    @validate_call(validate_return=True)
    async def delete(self, args: EnvDeleteArgs) -> None:
        value = args.model_dump()
        await self._rpc.call("envDelete", value)



import httpx
from pydantic import BaseModel
from pydantic_core import Url

from deno_sandbox.console import AsyncConsoleClient
from deno_sandbox.options import Options, get_internal_options


class VolumeCreateOptions(BaseModel):
    slug: str
    region: str
    capacity: int


class VolumeListOptions(BaseModel):
    cursor: str | None = None
    limit: int | None = None
    search: str | None = None


class Volume(BaseModel):
    id: str
    slug: str
    region: str
    capacity: int
    used: int


class AsyncVolumes:
    def __init__(self, options: Options | None = None):
        internal_options = get_internal_options(options or Options())
        self.__client = AsyncConsoleClient(internal_options)

    async def create(self, options: VolumeCreateOptions):
        json = await self.__client.post("/api/v2/volumes", options)
        return Volume(json)

    async def delete(self, id: str) -> None:
        await self.__client.delete(f"/api/v2/volumes/{id}")

    async def get(self, id: str) -> Volume | None:
        try:
            json = await self.__client.get(f"/api/v2/volumes/{id}")
            return Volume(json)
        except httpx.HTTPStatusError as e:
            if e.response.status_code == 404:
                return None
            raise

    async def list_all(self, options: VolumeListOptions) -> list[Volume]:
        url = Url("/api/v2/volumes")
        url.build_query(
            {
                "cursor": options.get("cursor", None),
                "limit": options.get("limit", None),
                "search": options.get("search", None),
            }
        )
        json = await self.__client.get(str(url))
        return [Volume(item) for item in json["volumes"]]



from abc import ABC, abstractmethod

from pydantic_core import Url
from websockets import ConnectionClosed, connect


class Transport(ABC):
    @abstractmethod
    async def send(self, data: dict[str, any]) -> None: ...

    @abstractmethod
    async def receive(self) -> dict[str, any]: ...

    @abstractmethod
    async def connect(self, url: Url, headers: dict[str, str]) -> None: ...

    @abstractmethod
    async def close(self) -> None: ...

    @abstractmethod
    async def __aiter__(self): ...


class WebSocketTransport(Transport):
    async def connect(self, url: Url, headers: dict[str, str]) -> None:
        self._ws = await connect(str(url), additional_headers=headers)

    async def send(self, data: str) -> None:
        await self._ws.send(data)

    async def receive(self) -> dict[str, any]:
        return await self._ws.recv()

    async def close(self) -> None:
        await self._ws.close()

    async def __aexit__(self):
        await self.close()

    async def __aiter__(self):
        try:
            async for message in self._ws:
                yield message
        except ConnectionClosed:
            return


class TransportFactory(ABC):
    @abstractmethod
    def create_transport(self) -> Transport: ...


class WebSocketTransportFactory(TransportFactory):
    def create_transport(self) -> Transport:
        return WebSocketTransport()



import asyncio
import threading


class AsyncBridge:
    def __init__(self):
        self.loop = asyncio.new_event_loop()
        self.thread = threading.Thread(target=self._run_loop, daemon=True)
        self.thread.start()

    def _run_loop(self):
        asyncio.set_event_loop(self.loop)
        self.loop.run_forever()

    def run(self, coro):
        """Submit a coroutine to the loop and wait for the result."""
        return asyncio.run_coroutine_threadsafe(coro, self.loop).result()

    def stop(self):
        self.loop.call_soon_threadsafe(self.loop.stop)
        self.thread.join()



from deno_sandbox.options import Options, get_internal_options
from deno_sandbox.sandbox import AsyncSandbox
from deno_sandbox.volumes import AsyncVolumes


class AsyncClient:
    def __init__(self, options: Options):
        self._options = get_internal_options(options or Options())
        self.sandbox = AsyncSandbox(options)
        self.volumes = AsyncVolumes(options)
